#include <avr/io.h>
#define F_CPU 16000000
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>
#include "u8g.h"

#define LCD_WIDTH  128
#define LCD_HEIGHT 64

#define PLAT_H 2
#define PLAT_W 12
#define MAN_H 6
#define MAN_W 6
#define FALL 0
#define JUMP 1
#define JUMP_H 40

typedef struct {
	int16_t x;
	int16_t y;
} point;

int16_t SCORE = 0;
char score[6];

u8g_t u8g;

point man;
int16_t xstep = 6;
int16_t ystep = 6;
int16_t state = 0;
point platforms[10];
int16_t ENERGY = JUMP_H;

/* Initialize inputs */
void init_buttons(void) {

    DDRA = 0xff;
    PORTA = 0x00;
	
	// Inputs
    DDRA &= ~(1 << PA4);
	DDRA &= ~(1 << PA5);
	DDRA &= ~(1 << PA6); 
    
	// Activate pull-up
	PORTA |= (1 << PA4);
    PORTA |= (1 << PA5);
    PORTA |= (1 << PA6);
}

void init_lcd(void) {
	MCUCR |= (1 << JTD); 
    MCUCR |= (1 << JTD);
    
    // init LCD
    u8g_Init8Bit(&u8g, &u8g_dev_st7920_128x64_4x_8bit,
            PN(1, 0), PN(1, 1), PN(1, 2), PN(1, 3), 
            PN(1, 4), PN(1, 5), PN(1, 6), PN(1, 7), // DATA PORTS
            PN(0, 2),    // ENABLE
            U8G_PIN_NONE,
            U8G_PIN_NONE,
            PN(0, 0),    // RS / DI
            PN(0, 1),    // RW
            PN(0, 3));   // RESET
}

void draw_score(void) {
    u8g_SetDefaultForegroundColor(&u8g);
    u8g_SetFont(&u8g, u8g_font_helvB08);
    u8g_SetFontPosCenter(&u8g);
	u8g_DrawStr90(&u8g, 70, 20, "YOU");
	u8g_DrawStr90(&u8g, 60, 18, "LOST");
	itoa(SCORE, score, 10);
	u8g_DrawStr90(&u8g, 46, 24, score);
	u8g_DrawStr90(&u8g, 36, 12, "POINTS");
    
}

void draw_man(void) {
	u8g_DrawBox(&u8g, man.x, man.y, 6, 6);
}

void read_buttons(void) {
		
	// LEFT
	if (!(PINA & (1 << PA4))) {
        man.y -= ystep;
		man.y %= (LCD_HEIGHT - 4); 
    }
    
    // RIGHT
    if (!(PINA & (1 << PA5))) {
        man.y += ystep;
		man.y %= (LCD_HEIGHT - 4); 
    }
}

void draw_platform(point p) {
	u8g_DrawBox(&u8g, p.x, p.y, PLAT_H, PLAT_W);
}

void detect_colisions(void) {
	int i = 0;
	point p;
	if (state == JUMP)
		return;
	for (; i < 10; i++) {
		p = platforms[i];
		if (man.y + MAN_W > p.y &&	// left
			man.y < p.y + PLAT_W && // right
			man.x + MAN_H > p.x && // up
			man.x <= p.x + PLAT_H) // down
		{
			man.x = p.x + PLAT_H;
			state = JUMP;
			ENERGY = JUMP_H;
			break;
		}
	}
}

void init_platforms(void) {
	int i = 0;
	for (; i < 10; i++)  {
		platforms[i].y = rand() % (LCD_HEIGHT - PLAT_H);
		platforms[i].x = rand() % (LCD_WIDTH - PLAT_W);
	}
}

void draw_platforms(void) {
	int i;
	for (i = 0; i < 10; i++) {
		draw_platform(platforms[i]);
	}
}

void update_man(void) {
	if (state == JUMP) {
		if (ENERGY > 0) {
			man.x += 5;
			ENERGY -= 5;
		} else {
			state = FALL;
		}
	} else if (state == FALL) {
		man.x -= 5;
	}
}

int16_t difficulty_parameter(void) {
	int16_t i = SCORE / 100;
	if (i >= 10)
		return 10;
	else if (i <= 1)
		return 2;
	else
		return i;
}

void update_platforms(void) {
	if (man.x > LCD_WIDTH / 2) {
		int16_t diff =  man.x - (LCD_WIDTH / 2);
		man.x -= diff;
		SCORE += diff;
		int i;
		for (i = 0; i < 10; i++) {
			platforms[i].x -= diff;
			if (platforms[i].x <= 2) {
				//platforms[i].x = LCD_WIDTH - 2 + platforms[i].x;
				platforms[i].x = LCD_WIDTH + (rand() % difficulty_parameter());
				platforms[i].y = rand() % (LCD_HEIGHT - PLAT_W);
			}
		}
	}
}

int main(void) {

    init_buttons();
    init_lcd();
	man.x = 50;
	man.y = 23;
	state = FALL;
	init_platforms();
	while(1) {
		read_buttons();
		update_man();
		detect_colisions();
		update_platforms();
		u8g_FirstPage(&u8g);
		do {
			draw_platforms();
			draw_man();
		} while (u8g_NextPage(&u8g));
		if (man.x < 4) {
			break;
		}
	}
	while(1) {
		u8g_FirstPage(&u8g);
		do {
			draw_score();
		} while (u8g_NextPage(&u8g));    
	}
return 0;
}
